#ifndef WALKER_H
#define WALKER_H

#include "SolverTypes.h"
#include "../mtl/Alg.h"
#include "../mtl/Heap.h"
#include "../mtl/Vec.h"
#include "../utils/Options.h"

#include "Random.h"

#include <vector>

class Random;

namespace Glucose {
  class Solver;

// Random walk local search based on 'ProbSAT' ideas.

class Walker {
public:
  Walker(Glucose::Solver *, double, int64_t, uint64_t);

  inline double score(unsigned i);
  CRef walk_pick_clause();
  Lit walk_pick_lit(const Clause&);

  Glucose::Solver *solver;
  Random random;
  int64_t propagations = 0; // local random number generator
  int64_t limit;        // limit on number of propagations
  std::vector<CRef> broken; // currently unsatisfied clauses
  int64_t broken_count = 0;
  double epsilon; // smallest considered score
  std::vector<double> table; // break value to score table

  std::vector<double> int_score; // scores of candidate literals
};

} // namespace Glucose

#endif
